package lib.ui.reactive;

import lib.Pos;
import util.Rect;
import util.ObservableArray;
import lib.Vec.Vec2;

class ListElement extends UIElement {

	private final content:ObservableArray<UIElement>;
	private var elementMap: Map<Rect,UIElement> = new Map(); // Position in the map is relative.
	private var offset:Pos;

	public function new(content: ObservableArray<UIElement>) {
		var events: UIEvents = {
			onClick: (p)->{
				var element = UIElement.getElementInMap((p.pos:Pos) - offset,elementMap);
				if (element != null)
					if (element.eventListner.onClick != null)
						element.eventListner.onClick.invoke(p);
			},
			onMouseUp: (p)->{
				var element = UIElement.getElementInMap((p.pos:Pos) - offset,elementMap);
				if (element != null)
					if (element.eventListner.onMouseUp != null)
						element.eventListner.onMouseUp.invoke(p);
			},
			onMouseScroll: (p)->{
				var element = UIElement.getElementInMap((p.pos:Pos) - offset,elementMap);
				if (element != null)
					if (element.eventListner.onMouseScroll != null)
						element.eventListner.onMouseScroll.invoke(p);
			},
		};

		super(events);

		this.content = content;
		this.content.subscribe(value -> {
			this.changedTrigger.trigger(null);
			// TODO: subscribe to elements and forward onChange event
		});
	}

	public function render(bounds:Vec2<Int>,offset: Pos):Canvas {
		var canvas: Canvas = new Canvas();
		var writePoint:Pos = {x: 0, y: 0};
		this.offset = offset;

		for(element in this.content.get()){
			if (bounds.y - writePoint.y <= 0) {
				// No more space to render children
				break;
			}

			var childRender = element.render({
				x: bounds.x,
				y: bounds.y - writePoint.y
			}, offset + writePoint);

			canvas.combine(childRender, writePoint);
			elementMap.set(new Rect(writePoint,
				{
					x: childRender.maxWidth() + writePoint.x,
					y: writePoint.y + (childRender.height() - 1),
				}
			),element);

			writePoint = {x: 0, y: writePoint.y + childRender.height()};
		}

		return canvas;
	}
}
